@@ -6,7 +6,8 @@
 #' @inheritParams sim_apply
 #' @param var either the name of a variable for which marginal effects are to be computed or a named list of length one containing the values the variable should take. If a list is supplied or the named variables is categorical (factor, character, or having two values), categorical calculations will be triggered. Otherwise, continuous calculations will be triggered. See Details.
 #' @param subset optional; a vector used to subset the data used to compute the marginal effects. This will be evaluated within the original dataset used to fit the model using [subset()], so nonstandard evaluation is allowed.
-#' @param contrast a string containing the name of a contrast between the average marginal means when the variable named in `var` is categorical and takes on two values. Allowed options include `"diff"` for the difference in means (also `"rd"`), `"rr"` for the risk ratio (also `"irr"`), `"log(rr):` for the log risk ratio (also `"log(irr)"`), `"or"` for the odds ratio, `"log(or)"` for the log odds ratio, and `"nnt"` for the number needed to treat. These options are not case sensitive, but the parentheses must be included if present.
+#' @param by a one-sided formula or character vector containing the names of variables for which to stratify the estimates. Each quantity will be computed within each level of the complete cross of the variables specified in `by`.
+#' @param contrast a string containing the name of a contrast between the average marginal means when the variable named in `var` is categorical and takes on two values. Allowed options include `"diff"` for the difference in means (also `"rd"`), `"rr"` for the risk ratio (also `"irr"`), `"log(rr):` for the log risk ratio (also `"log(irr)"`), `"sr"` for the survival ratio, `"log(sr):` for the log survival ratio, `"srr"` for the switch relative risk (also `"grrr"`), `"or"` for the odds ratio, `"log(or)"` for the log odds ratio, and `"nnt"` for the number needed to treat. These options are not case sensitive, but the parentheses must be included if present.
 #' @param outcome a string containing the name of the outcome or outcome level for multivariate (multiple outcomes) or multi-category outcomes. Ignored for univariate (single outcome) and binary outcomes.
 #' @param type a string containing the type of predicted values (e.g., the link or the response). Passed to [marginaleffects::get_predict()] and eventually to `predict()` in most cases. The default and allowable option depend on the type of model supplied, but almost always corresponds to the response scale (e.g., predicted probabilities for binomial models).
 #' @param eps when the variable named in `var` is continuous, the value by which to change the variable values to approximate the derivative. See Details.
@@ -27,18 +28,23 @@
 #' | --- | --- |
 #' |`"diff"`| `E[Y(1)] - E[Y(0)]` |
 #' |`"rr"`  | `E[Y(1)] / E[Y(0)]` |
-#' |`"or"`  | `O[Y(1)] / O[Y(0)]`, where `O[Y(.)]` = `E[Y(.)] / (1 - E[Y(.)])` |
+#' |`"sr"`  | `(1 - E[Y(1)]) / (1 - E[Y(0)])` |
+#' |`"srr"`  | `1 - sr` if  `E[Y(1)] > E[Y(0)]` |
+#' |        |   `rr - 1` if `E[Y(1)] < E[Y(0)]` |
+#' |        |  `0` otherwise |
+#' |`"or"`  | `O[Y(1)] / O[Y(0)]` |
+#' |        | where `O[Y(.)]` = `E[Y(.)] / (1 - E[Y(.)])` |
 #' |`"nnt"` | `1 / (E[Y(1)] - E[Y(0)])` |
 #'
 #' The `log(.)` versions are defined by taking the [log()] (natural log) of the corresponding effect measure.
 #'
 #' @return
 #' A `clarify_ame` object, which inherits from `clarify_est` and is similar to
-#' the output of `sim_apply()`, with the additional attribute `"var"` containing
-#' the variable named in `var`. The average marginal means will be named
+#' the output of `sim_apply()`, with the additional attributes `"var"` containing
+#' the variable named in `var` and `"by"` containing the names of the variables specified in `by` (if any). The average adjusted predictions will be named
 #' `E[Y({v})]`, where `{v}` is replaced with the values the focal variable
 #' (`var`) takes on. The average marginal effect for a continuous `var` will be
-#' named `dY/d({x})` where `{x}` is replaced with `var`.
+#' named `E[dY/d({x})]` where `{x}` is replaced with `var`. When `by` is specified, the average adjusted predictions will be named `E[Y({v})|{b}]` and the average marginel effect `E[dY/d({x})|{b}]` where `{b}` is a comma-separated list of of values of the `by` variables at which the quantity is computed. See examples.
 #'
 #' @seealso [sim_apply()], which provides a general interface to computing any
 #'   quantities for simulation-based inference; [plot.clarify_est()] for plotting the
@@ -51,7 +57,8 @@
 #' data("lalonde", package = "MatchIt")
 #'
 #' # Fit the model
-#' fit <- glm(I(re78 > 0) ~ treat + age + race + re74,
+#' fit <- glm(I(re78 > 0) ~ treat + age + race +
+#'              married + re74,
 #'            data = lalonde, family = binomial)
 #'
 #' # Simulate coefficients
@@ -62,12 +69,13 @@
 #' est <- sim_ame(s, var = "age", verbose = FALSE)
 #' summary(est)
 #'
-#' # Contrast between average marginal means for `treat`
+#' # Contrast between average adjusted predictions
+#' # for `treat`
 #' est <- sim_ame(s, var = "treat", contrast = "rr",
 #'                verbose = FALSE)
 #' summary(est)
 #'
-#' # Average marginal means for `race`; need to follow up
+#' # Average adjusted predictions for `race`; need to follow up
 #' # with contrasts for specific levels
 #' est <- sim_ame(s, var = "race", verbose = FALSE)
 #'
@@ -76,10 +84,41 @@
 #'
 #' summary(est)
 #'
+#' # Average adjusted predictions for `treat` within levels of
+#' # `married`, first using `subset` and then using `by`
+#' est0 <- sim_ame(s, var = "treat", subset = married == 0,
+#'                 contrast = "rd", verbose = FALSE)
+#' names(est0) <- paste0(names(est0), "|married=0")
+#' est1 <- sim_ame(s, var = "treat", subset = married == 1,
+#'                 contrast = "rd", verbose = FALSE)
+#' names(est1) <- paste0(names(est1), "|married=1")
+#'
+#' summary(cbind(est0, est1))
+#'
+#' est <- sim_ame(s, var = "treat", by = ~married,
+#'                contrast = "rd", verbose = FALSE)
+#'
+#' est
+#' summary(est)
+#'
+#' # Average marginal effect of `re74` within levels of
+#' # married*race
+#' est <- sim_ame(s, var = "age", by = ~married + race,
+#'                verbose = FALSE)
+#' est
+#' summary(est, null = 0)
+#'
+#' # Comparing AMEs between married and unmarried for
+#' # each level of `race`
+#' est_diff <- est[4:6] - est[1:3]
+#' names(est_diff) <- paste0("AME_diff|", levels(lalonde$race))
+#' summary(est_diff)
+#'
 #' @export
 sim_ame <- function(sim,
                     var,
                     subset = NULL,
+                    by = NULL,
                     contrast = NULL,
                     outcome = NULL,
                     type = NULL,
@@ -118,6 +157,15 @@ sim_ame <- function(sim,
                  var))
   }
 
+  if (!is.null(by)) {
+    if (is.character(by)) {
+      by <- reformulate(by)
+    }
+    else if (!inherits(by, "formula")) {
+      .err("`by` must be a one-sided formula or character vector")
+    }
+  }
+
   var_val <- dat[[var]]
   rm(dat)
 
@@ -134,16 +182,16 @@ sim_ame <- function(sim,
   }
 
   index.sub <- substitute(subset)
-  sim$fit <- attach_pred_data_to_fit(sim$fit, index.sub = index.sub,
+  sim$fit <- .attach_pred_data_to_fit(sim$fit, by = by, index.sub = index.sub,
                                      is_fitlist = is_misim)
 
   #Test to make sure compatible
   if (is_misim) {
-    test_dat <- get_pred_data_from_fit(sim$fit[[1]])
+    test_dat <- .get_pred_data_from_fit(sim$fit[[1]])
     test_predict <- clarify_predict(sim$fit[[1]], newdata = test_dat, group = NULL, type = type)
   }
   else {
-    test_dat <- get_pred_data_from_fit(sim$fit)
+    test_dat <- .get_pred_data_from_fit(sim$fit)
     test_predict <- clarify_predict(sim$fit, newdata = test_dat, group = NULL, type = type)
   }
 
@@ -155,11 +203,11 @@ sim_ame <- function(sim,
     if (!outcome %in% unique_group) {
       .err("only the following values of `outcome` are allowed: ", paste(add_quotes(unique_group), collapse = ", "))
     }
-    test_predict <- subset_group(test_predict, outcome)
+    test_predict <- .subset_group(test_predict, outcome)
   }
   else {
     if (!is.null(outcome)) {
-      chk::wrn("`outcome` is ignored for univariate models")
+      .wrn("`outcome` is ignored for univariate models")
     }
     outcome <- NULL
   }
@@ -180,79 +228,176 @@ sim_ame <- function(sim,
       if (!is.null(contrast)) {
         chk::chk_string(contrast)
         contrast <- tolower(contrast)
-        contrast <- match_arg(contrast, c("diff", "rd", "irr", "rr", "log(irr)",
+        contrast <- match_arg(contrast, c("diff", "rd", "irr", "rr", "sr", "srr", "grrr", "log(irr)",
                                           "log(rr)", "or", "log(or)", "nnt"))
       }
     }
     else if (!is.null(contrast)) {
-      chk::wrn("`contrast` is ignored when the focal variable takes on more than two levels")
+      .wrn("`contrast` is ignored when the focal variable takes on more than two levels")
       contrast <- NULL
     }
 
-    FUN <- function(fit) {
-      dat <- get_pred_data_from_fit(fit)
-      m <- nrow(dat)
-      dat2 <- do.call("rbind", lapply(vals, function(v) {
-        dat[[var]][] <- v
-        dat
-      }))
-      pred <- clarify_predict(fit, newdata = dat2, group = outcome, type = type)
-      p <- get_p(pred)
-      vapply(seq_along(vals), function(i) {
-        weighted.mean(p[seq_len(m) + (i - 1) * m], attr(fit, "weights"))
-      }, numeric(1L))
+    if (is.null(by)) {
+      FUN <- function(fit) {
+        dat <- .get_pred_data_from_fit(fit)
+        m <- nrow(dat)
+
+        dat2 <- dat[rep(seq_len(m), length(vals)),, drop = FALSE]
+        dat2[[var]][] <- rep(vals, each = m)
+
+        pred <- clarify_predict(fit, newdata = dat2, group = outcome, type = type)
+        p <- .get_p(pred)
+        vapply(seq_along(vals), function(i) {
+          weighted.mean(p[seq_len(m) + (i - 1) * m], attr(fit, "weights"))
+        }, numeric(1L))
+      }
+
+      out <- sim_apply(sim, FUN = FUN, verbose = verbose, cl = cl)
+
+      names(out) <- paste0("M", seq_along(vals))
+
+      if (!is.null(contrast)) {
+        out <- transform(out,
+                         `.C` = switch(tolower(contrast),
+                                       "diff" = , "rd" = M2 - M1,
+                                       "nnt" = 1 / (M2 - M1),
+                                       "irr" = , "rr" = M2 / M1,
+                                       "log(irr)" = , "log(rr)" = log(M2 / M1),
+                                       "sr" = (1 - M2) / (1 - M1),
+                                       "log(sr)" = log((1 - M2) / (1 - M1)),
+                                       "grrr" =, "srr" = (M2 == M1) * 0 +
+                                         (M2 > M1) * (1 - (1 - M2) / (1 - M1)) + (M2 < M1) * (M2 / M1 - 1),
+                                       "or" = (M2 / (1 - M2)) / (M1 / (1 - M1)),
+                                       "log(or)" = log((M2 / (1 - M2)) / (M1 / (1 - M1)))))
+        names(out)[3] <- .rename_contrast(contrast)
+      }
+
+      names(out)[seq_along(vals)] <- sprintf("E[Y(%s)]", vals)
     }
+    else {
+      FUN <- function(fit) {
+        dat <- .get_pred_data_from_fit(fit)
+        by_var <- .get_by_from_fit(fit)
+        m <- nrow(dat)
 
-    out <- sim_apply(sim, FUN = FUN, verbose = verbose, cl = cl)
+        dat2 <- dat[rep(seq_len(m), length(vals)),, drop = FALSE]
+        dat2[[var]][] <- rep(vals, each = m)
 
-    names(out) <- paste0("M", seq_along(vals))
+        pred <- clarify_predict(fit, newdata = dat2, group = outcome, type = type)
+        p <- .get_p(pred)
 
-    if (!is.null(contrast)) {
-      out <- transform(out,
-                       `.C` = switch(tolower(contrast),
-                                     "diff" = , "rd" = M2 - M1,
-                                     "nnt" = 1 / (M2 - M1),
-                                     "irr" = , "rr" = M2 / M1,
-                                     "log(irr)" = , "log(rr)" = log(M2 / M1),
-                                     "or" = (M2 / (1 - M2)) / (M1 / (1 - M1)),
-                                     "log(or)" = log((M2 / (1 - M2)) / (M1 / (1 - M1)))))
-      names(out)[3] <- rename_contrast(contrast)
+        unlist(lapply(levels(by_var), function(b) {
+          in_b <- by_var == b
+          w_b <- attr(fit, "weights")[in_b]
+          vapply(vals, function(v) {
+            weighted.mean(p[dat2[[var]] == v & in_b], w_b)
+          }, numeric(1L))
+        }))
+      }
+
+      out <- sim_apply(sim, FUN = FUN, verbose = verbose, cl = cl)
+
+      by_levels <- levels(.get_by_from_fit(sim$fit))
+
+      for (i in seq_along(by_levels)) {
+        names(out)[(i - 1) * length(vals) + seq_along(vals)] <- paste0("M", seq_along(vals))
+
+        if (!is.null(contrast)) {
+          out <- transform(out,
+                           `.C` = switch(tolower(contrast),
+                                         "diff" = , "rd" = M2 - M1,
+                                         "nnt" = 1 / (M2 - M1),
+                                         "irr" = , "rr" = M2 / M1,
+                                         "log(irr)" = , "log(rr)" = log(M2 / M1),
+                                         "sr" = (1 - M2) / (1 - M1),
+                                         "log(sr)" = log((1 - M2) / (1 - M1)),
+                                         "grrr" =, "srr" = (M2 > M1) * (1 - (1 - M2) / (1 - M1)) +
+                                           (M2 < M1) * (M2 / M1 - 1),
+                                         "or" = (M2 / (1 - M2)) / (M1 / (1 - M1)),
+                                         "log(or)" = log((M2 / (1 - M2)) / (M1 / (1 - M1)))))
+          names(out)[ncol(out)] <- sprintf("%s[%s]", .rename_contrast(contrast), by_levels[i])
+        }
+
+        names(out)[(i - 1) * length(vals) + seq_along(vals)] <- sprintf("E[Y(%s)|%s]", vals, by_levels[i])
+
+      }
+
+      if (!is.null(contrast)) {
+        #Re-order contrasts to be with by-levels
+        out <- out[unlist(lapply(seq_along(by_levels), function(i) {
+          c((i - 1) * length(vals) + seq_along(vals), length(by_levels) * length(vals) + i)
+        }))]
+      }
+
+      attr(out, "by") <- attr(sim$fit, "by_name")
     }
-
-    names(out)[seq_along(vals)] <- sprintf("E[Y(%s)]", vals)
-
   }
   else {
     chk::chk_number(eps)
     chk::chk_gt(eps)
 
     if (!is.null(contrast)) {
-      chk::wrn("`contrast` is ignored when the focal variable is continuous")
+      .wrn("`contrast` is ignored when the focal variable is continuous")
       contrast <- NULL
     }
 
     eps <- eps * sd(var_val)
 
-    FUN <- function(fit) {
-      dat <- get_pred_data_from_fit(fit)
-      ind <- seq_len(nrow(dat))
-      dat2 <- rbind(dat, dat)
+    if (is.null(by)) {
+      FUN <- function(fit) {
+        dat <- .get_pred_data_from_fit(fit)
+        ind <- seq_len(nrow(dat))
+        dat2 <- dat[c(ind, ind),, drop = FALSE]
 
-      dat2[[var]][ind] <- dat2[[var]][ind] - eps / 2
-      dat2[[var]][-ind] <- dat2[[var]][-ind] + eps / 2
+        dat2[[var]][ind] <- dat2[[var]][ind] - eps / 2
+        dat2[[var]][-ind] <- dat2[[var]][-ind] + eps / 2
 
-      pred <- clarify_predict(fit, newdata = dat2, group = outcome, type = type)
-      p <- get_p(pred)
+        pred <- clarify_predict(fit, newdata = dat2, group = outcome, type = type)
+        p <- .get_p(pred)
 
-      m0 <- weighted.mean(p[ind], attr(fit, "weights"))
-      m1 <- weighted.mean(p[-ind], attr(fit, "weights"))
+        m0 <- weighted.mean(p[ind], attr(fit, "weights"))
+        m1 <- weighted.mean(p[-ind], attr(fit, "weights"))
 
-      (m1 - m0) / eps
+        (m1 - m0) / eps
+      }
+
+      out <- sim_apply(sim, FUN = FUN, verbose = verbose, cl = cl)
+
+      names(out) <- sprintf("E[dY/d(%s)]", var)
     }
+    else {
+      FUN <- function(fit) {
+        dat <- .get_pred_data_from_fit(fit)
+        by_var <- .get_by_from_fit(fit)
+        ind <- seq_len(nrow(dat))
+        dat2 <- dat[c(ind, ind),, drop = FALSE]
 
-    out <- sim_apply(sim, FUN = FUN, verbose = verbose, cl = cl)
+        dat2[[var]][ind] <- dat2[[var]][ind] - eps / 2
+        dat2[[var]][-ind] <- dat2[[var]][-ind] + eps / 2
 
-    names(out) <- sprintf("dY/d(%s)", var)
+        pred <- clarify_predict(fit, newdata = dat2, group = outcome, type = type)
+        p <- .get_p(pred)
+
+        vapply(levels(by_var), function(b) {
+          in_b <- by_var == b
+          w_b <- attr(fit, "weights")[in_b]
+
+          m0 <- weighted.mean(p[ind][in_b], w_b)
+          m1 <- weighted.mean(p[-ind][in_b], w_b)
+
+          (m1 - m0) / eps
+        }, numeric(1L))
+
+      }
+
+      out <- sim_apply(sim, FUN = FUN, verbose = verbose, cl = cl)
+
+      by_levels <- levels(.get_by_from_fit(sim$fit))
+
+      names(out) <- sprintf("E[dY/d(%s)|%s]", var, by_levels)
+
+      attr(out, "by") <- attr(sim$fit, "by_name")
+    }
   }
 
   attr(out, "var") <- var
@@ -272,6 +417,9 @@ print.clarify_ame <- function(x, digits = NULL, max.ests = 6, ...) {
   cat("A `clarify_est` object (from `sim_ame()`)\n")
 
   cat(sprintf(" - Average marginal effect of `%s`\n", attr(x, "var")))
+  if (!is.null(attr(x, "by"))) {
+    cat(sprintf("   - within levels of %s\n", word_list(attr(x, "by"), quotes = "`")))
+  }
   cat(sprintf(" - %s simulated values\n", nrow(x)))
   cat(sprintf(" - %s %s estimated:", length(attr(x, "original")),
               ngettext(length(attr(x, "original")), "quantity", "quantities")))
@@ -286,7 +434,7 @@ print.clarify_ame <- function(x, digits = NULL, max.ests = 6, ...) {
   invisible(x)
 }
 
-rename_contrast <- function(x) {
+.rename_contrast <- function(x) {
   if (length(x) == 0) return(character(0))
   vapply(tolower(x), switch, character(1L),
          "diff" = "Diff",
@@ -296,9 +444,9 @@ rename_contrast <- function(x) {
          toupper(x))
 }
 
-attach_pred_data_to_fit <- function(fit, index.sub = NULL, is_fitlist = FALSE) {
+.attach_pred_data_to_fit <- function(fit, by = NULL, index.sub = NULL, is_fitlist = FALSE) {
   if (is_fitlist) {
-    return(lapply(fit, attach_pred_data_to_fit, index.sub))
+    return(lapply(fit, .attach_pred_data_to_fit, by = by, index.sub = index.sub))
   }
 
   data <- insight::get_data(fit, verbose = FALSE)
@@ -323,9 +471,19 @@ attach_pred_data_to_fit <- function(fit, index.sub = NULL, is_fitlist = FALSE) {
   attr(fit, "clarify_data") <- data[, intersect(vars, colnames(data)), drop = FALSE]
   attr(fit, "weights") <- weights
 
+  if (!is.null(by)) {
+    by_mf <- model.frame(update(by, NULL ~ .), data = data)
+    attr(fit, "by_var") <- factor(do.call("paste", c(as.list(by_mf), sep = ",")))
+    attr(fit, "by_name") <- names(by_mf)
+  }
+
   fit
 }
 
-get_pred_data_from_fit <- function(fit) {
+.get_pred_data_from_fit <- function(fit) {
   attr(fit, "clarify_data")
 }
+
+.get_by_from_fit <- function(fit) {
+  attr(fit, "by_var")
+}
