@@ -14,7 +14,7 @@ test_that("sim_ame() doesn't work with coefs and vcov", {
 test_that("sim_ame() works with lm()", {
   mdata <- readRDS(test_path("fixtures", "mdata.rds"))
 
-  fit <- lm(re78 ~ treat + age + educ + race + re74, data = mdata,
+  fit <- lm(re78 ~ treat * age + educ + race + re74, data = mdata,
             weights = weights)
 
   s <- sim(fit, n = 5)
@@ -62,7 +62,7 @@ test_that("sim_ame() works with lm()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   e <- sim_ame(s, "age", subset = treat == 1, verbose = FALSE)
@@ -87,6 +87,24 @@ test_that("sim_ame() works with lm()", {
   expect_error(sim_ame(s, list(0:1), verbose = FALSE),
                "desired focal variable or a named list")
 
+  #by
+  e <- sim_ame(s, "age", by = "treat", verbose = FALSE)
+  expect_good_clarify_est(e)
+  expect_equal(nrow(e), nrow(s$sim.coefs))
+  expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
+  expect_equal(ncol(e), length(unique(mdata$treat)))
+  expect_identical(names(e), c("E[dY/d(age)|0]", "E[dY/d(age)|1]"))
+  expect_identical(attr(e, "var"), "age")
+
+  e <- sim_ame(s, "treat", by = ~I(age < 30) + race, contrast = "diff", verbose = FALSE)
+  expect_good_clarify_est(e)
+  expect_equal(nrow(e), nrow(s$sim.coefs))
+  expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
+  expect_equal(ncol(e), 2 * nlevels(mdata$race) * (1 + length(unique(mdata$treat))))
+  expect_identical(names(e)[1:3], c("E[Y(0)|FALSE,black]", "E[Y(1)|FALSE,black]",
+                                    "Diff[FALSE,black]"))
+  expect_identical(attr(e, "var"), "treat")
+
 })
 
 test_that("sim_ame() works with glm()", {
@@ -140,7 +158,7 @@ test_that("sim_ame() works with glm()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   expect_warning(sim_ame(s, "age", contrast = "diff", verbose = FALSE),
@@ -219,7 +237,7 @@ test_that("sim_ame() works with MASS::glm.nb()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   e <- sim_ame(s, "age", subset = treat == 1, verbose = FALSE)
@@ -246,6 +264,105 @@ test_that("sim_ame() works with MASS::glm.nb()", {
 
 })
 
+test_that("sim_ame() works with MASS::polr()", {
+  skip_if_not_installed("MASS")
+  mdata <- readRDS(test_path("fixtures", "mdata.rds"))
+
+  suppressWarnings({
+    fit <- MASS::polr(ordY ~ treat + age + educ + race + re74,
+                      data = transform(mdata, ordY = ordered(countY)),
+                      weights = weights, Hess = TRUE)
+  })
+
+  s <- sim(fit, n = 5)
+
+  expect_error({
+    e <- sim_ame(s, "treat", verbose = FALSE)
+  }, "`outcome` must be supplied with multivariate models and models with multi-category outcomes.")
+
+  e <- sim_ame(s, "treat", outcome = "1", verbose = FALSE)
+
+  expect_good_clarify_est(e)
+  expect_equal(nrow(e), nrow(s$sim.coefs))
+  expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
+  expect_equal(ncol(e), 2)
+  expect_identical(names(e), c("E[Y(0)]", "E[Y(1)]"))
+  expect_identical(attr(e, "var"), "treat")
+
+  e <- sim_ame(s, "treat", contrast = "diff", outcome = "1", verbose = FALSE)
+
+  expect_good_clarify_est(e)
+  expect_equal(nrow(e), nrow(s$sim.coefs))
+  expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
+  expect_equal(ncol(e), 3)
+  expect_identical(names(e), c("E[Y(0)]", "E[Y(1)]", "Diff"))
+  expect_identical(attr(e, "var"), "treat")
+
+  expect_warning(sim_ame(s, "race", contrast = "diff", outcome = "1", verbose = FALSE),
+                 "`contrast` is ignored")
+
+  e <- sim_ame(s, "race", outcome = "1", verbose = FALSE)
+  expect_good_clarify_est(e)
+  expect_equal(nrow(e), nrow(s$sim.coefs))
+  expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
+  expect_equal(ncol(e), 3)
+  expect_identical(names(e), c("E[Y(black)]", "E[Y(hispan)]", "E[Y(white)]"))
+  expect_identical(attr(e, "var"), "race")
+
+  e <- sim_ame(s, list(race = c("black", "white")), contrast = "diff", outcome = "1", verbose = FALSE)
+  expect_good_clarify_est(e)
+  expect_equal(nrow(e), nrow(s$sim.coefs))
+  expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
+  expect_equal(ncol(e), 3)
+  expect_identical(names(e), c("E[Y(black)]", "E[Y(white)]", "Diff"))
+  expect_identical(attr(e, "var"), "race")
+
+  # Using `type = "mean"`
+  e <- sim_ame(s, "treat", contrast = "diff", type = "mean", verbose = FALSE)
+
+  expect_good_clarify_est(e)
+  expect_equal(nrow(e), nrow(s$sim.coefs))
+  expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
+  expect_equal(ncol(e), 3)
+  expect_identical(names(e), c("E[Y(0)]", "E[Y(1)]", "Diff"))
+  expect_identical(attr(e, "var"), "treat")
+
+  #Continuous variable
+  e <- sim_ame(s, "age", outcome = "1", verbose = FALSE)
+  expect_good_clarify_est(e)
+  expect_equal(nrow(e), nrow(s$sim.coefs))
+  expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
+  expect_equal(ncol(e), 1)
+  expect_identical(names(e), "E[dY/d(age)]")
+  expect_identical(attr(e, "var"), "age")
+
+  e <- sim_ame(s, "age", outcome = "1", subset = treat == 1, verbose = FALSE)
+  expect_good_clarify_est(e)
+
+  expect_warning(sim_ame(s, "age", outcome = "1", contrast = "diff", verbose = FALSE),
+                 "`contrast` is ignored")
+
+  e <- sim_ame(s, "age", type = "mean", subset = treat == 1, verbose = FALSE)
+  expect_good_clarify_est(e)
+
+  #Bad args
+  expect_error(sim_ame(s, list(race = c("black", "whiteAAA")), verbose = FALSE),
+               "values mentioned in")
+  expect_error(sim_ame(s, list(race = 1:2), verbose = FALSE),
+               "values mentioned in")
+  expect_error(sim_ame(s, list(raceAAA = 1:2), verbose = FALSE),
+               "not present in the original model")
+  expect_error(sim_ame(s, "raceAAA", verbose = FALSE),
+               "not present in the original model")
+  expect_error(sim_ame(s, c("race", "treat"), verbose = FALSE),
+               "desired focal variable or a named list")
+  expect_error(sim_ame(s, list(race = "black", treat = 0:1), verbose = FALSE),
+               "desired focal variable or a named list")
+  expect_error(sim_ame(s, list(0:1), verbose = FALSE),
+               "desired focal variable or a named list")
+
+})
+
 test_that("sim_ame() works with betareg::betareg()", {
   skip_if_not_installed("betareg")
   mdata <- readRDS(test_path("fixtures", "mdata.rds"))
@@ -298,7 +415,7 @@ test_that("sim_ame() works with betareg::betareg()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   e <- sim_ame(s, "age", subset = treat == 1, verbose = FALSE)
@@ -377,7 +494,7 @@ test_that("sim_ame() works with survey::svyglm()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   expect_warning(sim_ame(s, "age", contrast = "diff", verbose = FALSE),
@@ -456,7 +573,7 @@ test_that("sim_ame() works with estimatr::lm_robust()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   e <- sim_ame(s, "age", subset = treat == 1, verbose = FALSE)
@@ -536,7 +653,7 @@ test_that("sim_ame() works with estimatr::iv_robust()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   e <- sim_ame(s, "age", subset = treat == 1, verbose = FALSE)
@@ -617,7 +734,7 @@ test_that("sim_ame() works with fixest::feols()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   e <- sim_ame(s, "age", subset = treat == 1, verbose = FALSE)
@@ -698,7 +815,7 @@ test_that("sim_ame() works with fixest::feglm()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   expect_warning(sim_ame(s, "age", contrast = "diff", verbose = FALSE),
@@ -777,7 +894,7 @@ test_that("sim_ame() works with logistf::logistf()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   expect_warning(sim_ame(s, "age", contrast = "diff", verbose = FALSE),
@@ -860,7 +977,7 @@ test_that("sim_ame() works with geepack::geeglm()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   expect_warning(sim_ame(s, "age", contrast = "diff", verbose = FALSE),
@@ -945,7 +1062,7 @@ test_that("sim_ame() works with rms::ols()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   e <- sim_ame(s, "age", subset = treat == 1, verbose = FALSE)
@@ -1032,7 +1149,7 @@ test_that("sim_ame() works with rms::lrm()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   expect_warning(sim_ame(s, "age", contrast = "diff", verbose = FALSE),
@@ -1113,7 +1230,7 @@ test_that("sim_ame() works with robustbase::lmrob()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   e <- sim_ame(s, "age", subset = treat == 1, verbose = FALSE)
@@ -1193,7 +1310,7 @@ test_that("sim_ame() works with robustbase::glmrob()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   expect_warning(sim_ame(s, "age", contrast = "diff", verbose = FALSE),
@@ -1272,7 +1389,7 @@ test_that("sim_ame() works with robust::lmRob()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   e <- sim_ame(s, "age", subset = treat == 1, verbose = FALSE)
@@ -1354,7 +1471,7 @@ test_that("sim_ame() works with robust::glmRob()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   expect_warning(sim_ame(s, "age", contrast = "diff", verbose = FALSE),
@@ -1433,7 +1550,7 @@ test_that("sim_ame() works with AER::tobit()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   e <- sim_ame(s, "age", subset = treat == 1, verbose = FALSE)
@@ -1512,7 +1629,7 @@ test_that("sim_ame() works with ivreg::ivreg()", {
   expect_equal(nrow(e), nrow(s$sim.coefs))
   expect_equal(attr(e, "sim_hash"), attr(s, "sim_hash"))
   expect_equal(ncol(e), 1)
-  expect_identical(names(e), "dY/d(age)")
+  expect_identical(names(e), "E[dY/d(age)]")
   expect_identical(attr(e, "var"), "age")
 
   e <- sim_ame(s, "age", subset = treat == 1, verbose = FALSE)
